@@ -6,7 +6,7 @@
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclUnixCompat.c,v 1.15 2008/02/28 20:14:12 jenglish Exp $
+ * RCS: @(#) $Id: tclUnixCompat.c,v 1.15.2.1 2010/01/06 21:35:22 nijtmans Exp $
  *
  */
 
@@ -127,7 +127,7 @@ static int		CopyGrp(struct group *tgtPtr, char *buf, int buflen);
 static int		CopyHostent(struct hostent *tgtPtr, char *buf,
 			    int buflen);
 static int		CopyPwd(struct passwd *tgtPtr, char *buf, int buflen);
-static int		CopyString(char *src, char *buf, int buflen);
+static int		CopyString(CONST char *src, char *buf, int buflen);
 
 #endif
 #endif /* TCL_THREADS */
@@ -736,7 +736,7 @@ CopyArray(
     p = buf + len;
 
     for (j = 0; j < i; j++) {
-	int sz = (elsize<0 ? strlen(src[j])+1 : elsize);
+	int sz = (elsize<0 ? (int) strlen(src[j]) + 1 : elsize);
 
 	len += sz;
 	if (len > buflen) {
@@ -772,7 +772,7 @@ CopyArray(
 #ifdef NEED_COPYSTRING
 static int
 CopyString(
-    char *src,			/* String to copy. */
+    CONST char *src,	/* String to copy. */
     char *buf,			/* Buffer to copy into. */
     int buflen)			/* Size of buffer. */
 {
