@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclUnixNotfy.c,v 1.34 2008/03/11 22:23:50 das Exp $
+ * RCS: @(#) $Id: tclUnixNotfy.c,v 1.34.2.2 2010/01/29 09:38:47 nijtmans Exp $
  */
 
 #include "tclInt.h"
@@ -143,7 +143,7 @@ static ThreadSpecificData *waitingListPtr = NULL;
  * pipe. Hence writing to this file descriptor will cause the select() system
  * call to return and wake up the notifier thread.
  *
- * You must hold the notifierMutex lock before accessing this list.
+ * You must hold the notifierMutex lock before writing to the pipe.
  */
 
 static int triggerPipe = -1;
@@ -199,7 +199,7 @@ static int	FileHandlerEventProc(Tcl_Event *evPtr, int flags);
  *	Initializes the platform specific notifier state.
  *
  * Results:
- *	Returns a handle to the notifier state for this thread..
+ *	Returns a handle to the notifier state for this thread.
  *
  * Side effects:
  *	None.
@@ -292,7 +292,9 @@ Tcl_FinalizeNotifier(
 	 * 4139] [Bug: 1222872]
 	 */
 
-	write(triggerPipe, "q", 1);
+	if (write(triggerPipe, "q", 1) != 1) {
+		    Tcl_Panic("Tcl_FinalizeNotifier: unable to write q to triggerPipe");
+	}
 	close(triggerPipe);
 	while(triggerPipe >= 0) {
 	    Tcl_ConditionWait(&notifierCV, &notifierMutex, NULL);
@@ -543,15 +545,17 @@ Tcl_DeleteFileHandler(
      */
 
     if (fd+1 == tsdPtr->numFdBits) {
-	tsdPtr->numFdBits = 0;
+	int numFdBits = 0;
+
 	for (i = fd-1; i >= 0; i--) {
 	    if (FD_ISSET(i, &(tsdPtr->checkMasks.readable))
 		    || FD_ISSET(i, &(tsdPtr->checkMasks.writable))
 		    || FD_ISSET(i, &(tsdPtr->checkMasks.exceptional))) {
-		tsdPtr->numFdBits = i+1;
+		numFdBits = i+1;
 		break;
 	    }
 	}
+	tsdPtr->numFdBits = numFdBits;
     }
 
     /*
@@ -664,10 +668,9 @@ Tcl_WaitForEvent(
     FileHandler *filePtr;
     FileHandlerEvent *fileEvPtr;
     int mask;
-    Tcl_Time myTime;
+    Tcl_Time vTime;
 #ifdef TCL_THREADS
     int waitForFiles;
-    Tcl_Time *myTimePtr;
 #else
     /*
      * Impl. notes: timeout & timeoutPtr are used if, and only if threads are
@@ -696,22 +699,15 @@ Tcl_WaitForEvent(
 	 * handler to do this scaling.
 	 */
 
-	myTime.sec  = timePtr->sec;
-	myTime.usec = timePtr->usec;
-
-	if (myTime.sec != 0 || myTime.usec != 0) {
-	    (*tclScaleTimeProcPtr) (&myTime, tclTimeClientData);
+	if (timePtr->sec != 0 || timePtr->usec != 0) {
+	    vTime = *timePtr;
+	    (*tclScaleTimeProcPtr) (&vTime, tclTimeClientData);
+	    timePtr = &vTime;
 	}
-
-#ifdef TCL_THREADS
-	myTimePtr = &myTime;
-#else
-	timeout.tv_sec = myTime.sec;
-	timeout.tv_usec = myTime.usec;
-	timeoutPtr = &timeout;
-#endif /* TCL_THREADS */
-
 #ifndef TCL_THREADS
+	timeout.tv_sec = timePtr->sec;
+	timeout.tv_usec = timePtr->usec;
+	timeoutPtr = &timeout;
     } else if (tsdPtr->numFdBits == 0) {
 	/*
 	 * If there are no threads, no timeout, and no fds registered, then
@@ -722,11 +718,7 @@ Tcl_WaitForEvent(
 	 */
 
 	return -1;
-#endif /* !TCL_THREADS */
     } else {
-#ifdef TCL_THREADS
-	myTimePtr = NULL;
-#else
 	timeoutPtr = NULL;
 #endif /* TCL_THREADS */
     }
@@ -739,8 +731,7 @@ Tcl_WaitForEvent(
 
     Tcl_MutexLock(&notifierMutex);
 
-    waitForFiles = (tsdPtr->numFdBits > 0);
-    if (myTimePtr != NULL && myTimePtr->sec == 0 && (myTimePtr->usec == 0
+    if (timePtr != NULL && timePtr->sec == 0 && (timePtr->usec == 0
 #if defined(__APPLE__) && defined(__LP64__)
 	    /*
 	     * On 64-bit Darwin, pthread_cond_timedwait() appears to have a bug
@@ -748,7 +739,7 @@ Tcl_WaitForEvent(
 	     * has already been exceeded by the system time; as a workaround,
 	     * when given a very brief timeout, just do a poll. [Bug 1457797]
 	     */
-	    || myTimePtr->usec < 10
+	    || timePtr->usec < 10
 #endif
 	    )) {
 	/*
@@ -761,8 +752,9 @@ Tcl_WaitForEvent(
 
 	waitForFiles = 1;
 	tsdPtr->pollState = POLL_WANT;
-	myTimePtr = NULL;
+	timePtr = NULL;
     } else {
+	waitForFiles = (tsdPtr->numFdBits > 0);
 	tsdPtr->pollState = 0;
     }
 
@@ -781,7 +773,9 @@ Tcl_WaitForEvent(
 	waitingListPtr = tsdPtr;
 	tsdPtr->onList = 1;
 
-	write(triggerPipe, "", 1);
+	if (write(triggerPipe, "", 1) != 1) {
+		Tcl_Panic("Tcl_WaitForEvent: unable to write to triggerPipe");
+	}
     }
 
     FD_ZERO(&(tsdPtr->readyMasks.readable));
@@ -789,7 +783,7 @@ Tcl_WaitForEvent(
     FD_ZERO(&(tsdPtr->readyMasks.exceptional));
 
     if (!tsdPtr->eventReady) {
-	Tcl_ConditionWait(&tsdPtr->waitCV, &notifierMutex, myTimePtr);
+	Tcl_ConditionWait(&tsdPtr->waitCV, &notifierMutex, timePtr);
     }
     tsdPtr->eventReady = 0;
 
@@ -811,7 +805,9 @@ Tcl_WaitForEvent(
 	}
 	tsdPtr->nextPtr = tsdPtr->prevPtr = NULL;
 	tsdPtr->onList = 0;
-	write(triggerPipe, "", 1);
+	if (write(triggerPipe, "", 1) != 1) {
+		Tcl_Panic("Tcl_WaitForEvent: unable to write to triggerPipe");
+	}
     }
 
 #else
