@@ -9,13 +9,11 @@
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinThrd.c,v 1.43 2007/03/24 09:33:02 vasiljevic Exp $
+ * RCS: @(#) $Id: tclWinThrd.c,v 1.43.4.2 2010/01/31 23:51:37 nijtmans Exp $
  */
 
 #include "tclWinInt.h"
 
-#include <fcntl.h>
-#include <io.h>
 #include <sys/stat.h>
 
 /*
@@ -43,8 +41,10 @@ static CRITICAL_SECTION initLock;
 
 #ifdef TCL_THREADS
 
-static CRITICAL_SECTION allocLock;
-static Tcl_Mutex allocLockPtr = (Tcl_Mutex) &allocLock;
+static struct Tcl_Mutex_ {
+    CRITICAL_SECTION crit;
+} allocLock;
+static Tcl_Mutex allocLockPtr = &allocLock;
 static int allocOnce = 0;
 
 #endif /* TCL_THREADS */
@@ -154,6 +154,10 @@ TclpThreadCreate(
 
     EnterCriticalSection(&joinLock);
 
+    *idPtr = 0; /* must initialize as Tcl_Thread is a pointer and
+                 * on WIN64 sizeof void* != sizeof unsigned
+		 */
+
 #if defined(_MSC_VER) || defined(__MSVCRT__) || defined(__BORLANDC__)
     tHandle = (HANDLE) _beginthreadex(NULL, (unsigned) stackSize, proc,
 	    clientData, 0, (unsigned *)idPtr);
@@ -408,7 +412,7 @@ Tcl_GetAllocMutex(void)
 {
 #ifdef TCL_THREADS
     if (!allocOnce) {
-	InitializeCriticalSection(&allocLock);
+	InitializeCriticalSection(&allocLock.crit);
 	allocOnce = 1;
     }
     return &allocLockPtr;
@@ -450,7 +454,7 @@ TclFinalizeLock(void)
 
 #ifdef TCL_THREADS
     if (allocOnce) {
-	DeleteCriticalSection(&allocLock);
+	DeleteCriticalSection(&allocLock.crit);
 	allocOnce = 0;
     }
 #endif
@@ -491,6 +495,7 @@ Tcl_MutexLock(
     Tcl_Mutex *mutexPtr)	/* The lock */
 {
     CRITICAL_SECTION *csPtr;
+
     if (*mutexPtr == NULL) {
 	MASTER_LOCK;
 
@@ -531,6 +536,7 @@ Tcl_MutexUnlock(
     Tcl_Mutex *mutexPtr)	/* The lock */
 {
     CRITICAL_SECTION *csPtr = *((CRITICAL_SECTION **)mutexPtr);
+
     LeaveCriticalSection(csPtr);
 }
 
@@ -556,6 +562,7 @@ TclpFinalizeMutex(
     Tcl_Mutex *mutexPtr)
 {
     CRITICAL_SECTION *csPtr = *(CRITICAL_SECTION **)mutexPtr;
+
     if (csPtr != NULL) {
 	DeleteCriticalSection(csPtr);
 	ckfree((char *) csPtr);
@@ -641,11 +648,11 @@ Tcl_ConditionWait(
 	 */
 
 	if (*condPtr == NULL) {
-	    winCondPtr = (WinCondition *)ckalloc(sizeof(WinCondition));
+	    winCondPtr = (WinCondition *) ckalloc(sizeof(WinCondition));
 	    InitializeCriticalSection(&winCondPtr->condLock);
 	    winCondPtr->firstPtr = NULL;
 	    winCondPtr->lastPtr = NULL;
-	    *condPtr = (Tcl_Condition)winCondPtr;
+	    *condPtr = (Tcl_Condition) winCondPtr;
 	    TclRememberCondition(condPtr);
 	}
 	MASTER_UNLOCK;
@@ -755,6 +762,7 @@ Tcl_ConditionNotify(
 {
     WinCondition *winCondPtr;
     ThreadSpecificData *tsdPtr;
+
     if (*condPtr != NULL) {
 	winCondPtr = *((WinCondition **)condPtr);
 
@@ -811,6 +819,7 @@ FinalizeConditionEvent(
     ClientData data)
 {
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *) data;
+
     tsdPtr->flags = WIN_THREAD_UNINIT;
     CloseHandle(tsdPtr->condEvent);
 }
@@ -854,6 +863,9 @@ TclpFinalizeCondition(
     }
 }
 
+
+
+
 /*
  * Additions by AOL for specialized thread memory allocator.
  */
